/*\
 *
 *

    MIT License

    Copyright (c) 2020 Yves BAZIN

    Permission is hereby granted, free of charge, to any person obtaining a copy of
    this software and associated documentation files (the "Software"), to deal in
    the Software without restriction, including without limitation the rights to
    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is furnished to do
    so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

 *
 *
\*/

#ifndef SID6581_h
#define SID6581_h

#define CS 2
#define WRITE 1
#define CS_1 1
#define WRITE_1 0

#define CS_2 3
#define WRITE_2 2

#define CS_3 5
#define WRITE_3 4

#define CS_4 7
#define WRITE_4 6

#define RESET 0
#define MASK_ADDRESS 0b11111000
#define MASK_CSWRRE 0b111
#define SID_WAVEFORM_TRIANGLE (1<<4)
#define SID_WAVEFORM_SAWTOOTH (1<<5)
#define SID_WAVEFORM_PULSE (1<<6)
#define SID_WAVEFORM_NOISE (1<<7)
#define SID_WAVEFORM_SILENCE 0
#define SID_GATE 0
#define SID_TEST (1<<3)
#define SID_SYNC 1
#define SID_RINGMODE (1<<2)
#define SID_3OFF (1<<7)
#define SID_HP (1<<6)
#define SID_BP (1<<5)
#define SID_LP (1<<4)
#define SID_FILT1 1
#define SID_FILT2 2
#define SID_FILT3 4
#define SID_FILTEX 8
#define SID_FREQ_LO 0
#define SID_FREQ_HI 1
#define SID_PW_LO 2
#define SID_PW_HI 3
#define SID_CONTROL_REG 4
#define SID_ATTACK_DECAY 5
#define SID_SUSTAIN_RELEASE 6

#define SID_FREQ_LO_0 0
#define SID_FREQ_HI_0 1
#define SID_PW_LO_0 2
#define SID_PW_HI_0 3
#define SID_CONTROL_REG_0 4
#define SID_ATTACK_DECAY_0 5
#define SID_SUSTAIN_RELEASE_0 6

#define SID_FREQ_LO_1 7
#define SID_FREQ_HI_1 8
#define SID_PW_LO_1 9
#define SID_PW_HI_1 10
#define SID_CONTROL_REG_1 11
#define SID_ATTACK_DECAY_1 12
#define SID_SUSTAIN_RELEASE_1 13

#define SID_FREQ_LO_2 14
#define SID_FREQ_HI_2 15
#define SID_PW_LO_2 16
#define SID_PW_HI_2 17
#define SID_CONTROL_REG_2 18
#define SID_ATTACK_DECAY_2 19
#define SID_SUSTAIN_RELEASE_2 20



#define SID_FC_LO 21
#define SID_FC_HI 22
#define SID_RES_FILT 23
#define SID_MOD_VOL 24


#define SID_QUEUE_SIZE 2000


#include "SPI.h"
#include "FS.h"
#include "soc/timer_group_struct.h"
#include "soc/timer_group_reg.h"
#include "driver/i2s.h"
#include "freertos/queue.h"
#include "Sid_md5.hpp"

static TaskHandle_t xPlayerTaskHandle = NULL;
static TaskHandle_t SIDPlayerTaskHandle = NULL;
static TaskHandle_t SIDPlayerLoopTaskHandle = NULL;
static TaskHandle_t PausePlayTaskLocker = NULL;
static TaskHandle_t xPushToRegisterHandle = NULL;
static QueueHandle_t _sid_queue;
static QueueHandle_t _sid_voicesQueues[15];
static uint16_t _sid_play_notes[96];
static TaskHandle_t _sid_xHandletab[15];
static uint8_t keyboardnbofvoices;
volatile static bool _sid_voices_busy[15];
volatile static bool _sid_taskready_busy[15];
static int _sid_sustain_decay_durations [16]={6,24,48,72,114,168,204,240,300,750,1500,2400,3000,9000,15000,24000};
//to save the


struct _sid_register_to_send{
    uint8_t data;
    uint8_t chip;
    uint8_t address;
};


struct _sid_voice {
    uint16_t frequency;
    uint16_t pulse;
    uint8_t waveform;
    uint8_t attack;
    uint8_t decay;
    uint8_t sustain;
    uint8_t release;
    uint8_t gate;
    uint8_t sync;
    uint8_t test;
    uint8_t ringmode;
    uint8_t freq_lo,freq_hi,pw_lo,pw_hi,control_reg,attack_decay,sustain_release;
};


struct _sid_control {
    uint8_t volume;
    uint8_t filterfrequency;
    uint8_t res;
    uint8_t filt1,filt2,filt3,filtex;
    uint8_t _3off,hp,bp,lp;
    uint8_t fc_lo,fc_hi,res_filt,mode_vol;
};


struct songstruct{
    fs::FS  *fs;
    char filename[80];
    uint8_t name[32];
    uint8_t author[32];
    char md5[32];
    uint8_t published[32];
    uint8_t subsongs,startsong;
    uint32_t  durations[32];
};


struct _sid_command {
    int note;
    int velocity;
    uint16_t duration;
};

enum sid_error{
    NOT_ENOUGH_MEMORY,

};


enum sidEvent {
    SID_NEW_TRACK,
    SID_NEW_FILE,
    SID_START_PLAY,
    SID_END_PLAY,
    SID_PAUSE_PLAY,
    SID_RESUME_PLAY,
    SID_END_TRACK,
    SID_STOP_TRACK
};


class SID6581 {
  public:
    SID6581();
    bool begin(int clock_pin,int data_pin, int latch );
    bool begin(int clock_pin,int data_pin, int latch,int sid_clock_pin);
  /*
    void play();
    void playNext();
    void playPrev();
    void soundOff();
    void soundOn();
    void pausePlay();
    void sidSetMaxVolume( uint8_t volume);
    void addSong(fs::FS &fs,  const char * path);
    void playSongNumber(int number);
    void stopPlayer();
    void setVoice(uint8_t vo);
    */

    void sidSetVolume( int chip,uint8_t vol);
    void setFrequency(int voice, uint16_t frequency);
    void setFrequencyHz(int voice,double frequencyHz);
    void setPulse(int voice, uint16_t pulse);
    void setEnv(int voice, uint8_t att,uint8_t decay,uint8_t sutain, uint8_t release);
    void setAttack(int voice, uint8_t att);
    void setDecay(int voice, uint8_t decay);
    void setSustain(int voice,uint8_t sutain);
    void setRelease(int voice,uint8_t release);
    void setGate(int voice, int gate);
    void setWaveForm(int voice,int waveform);
    void setTest(int voice,int test);
    void setSync(int voice,int sync);
    void setRingMode(int voice, int ringmode);
    void setFilterFrequency(int chip,int filterfrequency);
    void setResonance(int chip,int resonance);
    void setFilter1(int chip,int filt1);
    void setFilter2(int chip,int filt2);
    void setFilter3(int chip,int filt3);
    void setFilterEX(int chip,int filtex);
    void set3OFF(int chip,int _3off);
    void setHP(int chip,int hp);
    void setBP(int chip,int bp);
    void setLP(int chip,int lp);

    void pushToVoice(int voice,uint8_t address,uint8_t data);
    void pushRegister(int chip,int address,uint8_t data);
    void resetsid();
    void feedTheDog();



    int getSidVolume( int chip);
    int getFrequency(int voice);
    double getFrequencyHz(int voice);
    int getPulse(int voice);
    int getAttack(int voice);
    int getDecay(int voice);
    int getSustain(int voice);
    int getRelease(int voice);
    int getGate(int voice);
    int getWaveForm(int voice);
    int getTest(int voice);
    int getSync(int voice);
    int getRingMode(int voice);
    int getFilterFrequency(int chip);
    int getResonance(int chip);
    int getFilter1(int chip);
    int getFilter2(int chip);
    int getFilter3(int chip);
    int getFilterEX(int chip);
    int get3OFF(int chip);
    int getHP(int chip);
    int getBP(int chip);
    int getLP(int chip);

    int getRegister(int chip,int addr);

    _sid_voice voices[15];
    uint8_t sidregisters[15*32];
    void clearcsw(int chip);
    //void resetsid();
    void push();

    void setA(uint8_t val);
    void setcsw();
    void setD(uint8_t val);
    void cls();
    void soundOn();
    void soundOff();


  protected:

    int  latch_pin;
     Sid_md5 md5;
    //void stop();
    //uint8_t voice=7;
    int saveVolume[5];
    _sid_control sid_control[5];
    void playNextInt();
    /*bool stopped=false;
    int numberToPlay=0;
    int numberOfSongs=0;
    int currentSong=0;
    bool paused=false;*/
    //  const int _maxnumber=80;
    //static void playSIDTunesTask(void *pvParameters);
    static void _pushRegister(void *pvParameters);
    /* static void playSIDTunesLoopTask(void *pvParameters);
    songstruct listsongs[80];
    uint8_t save24;
    uint8_t volume;
    uint8_t * sidvalues;
    unsigned int readFile2(fs::FS &fs, const char * path);*/
    uint8_t adcswrre,dataspi,chipcontrol;
    SPIClass * sid_spi = NULL;
    const int sid_spiClk = 20000000;
};


static   SID6581 sid;



class SIDRegisterPlayer {
  public:

    void play();
    void playNext();
    void playPrev();
    //void soundOff();
    //void soundOn();
    void pausePlay();
    void SetMaxVolume( uint8_t volume);
    void addSong(fs::FS &fs,  const char * path);
    void playSongNumber(int number);
    void stopPlayer();
    void setVoice(uint8_t vo);
    bool begin(int clock_pin,int data_pin, int latch );
    bool begin(int clock_pin,int data_pin, int latch,int sid_clock_pin);
    char * getFilename();
    int getPositionInPlaylist();
    int getPlaylistSize();
    void executeEventCallback(sidEvent event);
    inline void setEventCallback(void (*fptr)(sidEvent event)) {
        eventCallback = fptr;
    }

  private:
    char return_filename[250];
    uint8_t voice=7;
    const int _maxnumber=255;
    void playNextInt();
    bool stopped=false;
    int numberToPlay=0;
    int numberOfSongs=0;
    int currentSong=0;
    bool paused=false;
    void stop();
    static void playSIDTunesTask(void *pvParameters);
    static void playSIDTunesLoopTask(void *pvParameters);
    songstruct listsongs[255];
    uint8_t save24;
    uint8_t volume;
    uint8_t * sidvalues;
    void (*eventCallback)(sidEvent event)=NULL;
    unsigned int readFile2(fs::FS &fs, const char * path);
};


//static SIDRegisterPlayer sidRegisterPlayer;


class sid_instrument{
  public:
    int attack;
    int sustain;
    int decay;
    int release;
    int pulse;
    int waveform;
    int type_instrument;
    int current_note;
    uint32_t sustain_time;
    virtual void start_sample(int voice,int note){}
    virtual void next_instruction(int voice,int note){}
    virtual void after_off(int voice,int note){}
    virtual void after_offinstrument(int voice, int note) {
      //
    }
    int start;
    int loop;
};


static sid_instrument *current_instruments[9];


template<int voice>
class VoiceTask{
  public:
    static void  vTaskCode( void * pvParameters ) {
        _sid_taskready_busy[voice]=true;
        uint32_t start_time;
        uint32_t sustain_time;
        _sid_command element;
        for( ;; ) {
            xQueueReceive(_sid_voicesQueues[voice], &element, portMAX_DELAY);
            if(element.velocity>0) {
                current_instruments[voice]->release=0;
                current_instruments[voice]->current_note=element.note;
                current_instruments[voice]->start_sample(voice,element.note);
                current_instruments[voice]->release=sid.voices[voice].release;
            }
            start_time=millis();
            while(uxQueueMessagesWaiting( _sid_voicesQueues[voice] )==0) {
                if(element.duration>0) {
                    if(millis()-start_time>=element.duration) {
                        sustain_time=millis();
                        sid.setGate(voice,0);
                        while(uxQueueMessagesWaiting( _sid_voicesQueues[voice] )==0) {
                            if(millis()-sustain_time>=_sid_sustain_decay_durations[current_instruments[voice]->release]) {
                                sid.setFrequency(voice,0);
                                _sid_voices_busy[voice]=false;
                                break;
                            } else {
                                current_instruments[voice]->after_off(voice,current_instruments[voice]->current_note);
                                vTaskDelay(1);
                            }
                        }
                        _sid_voices_busy[voice]=false;
                        break;
                    }
                }
                if(element.velocity>0) {
                    current_instruments[voice]->next_instruction(voice,element.note);
                    vTaskDelay(1);
                } else {
                    sustain_time=millis();
                    sid.setGate(voice,0);
                    _sid_voices_busy[voice]=false;
                    while(uxQueueMessagesWaiting( _sid_voicesQueues[voice] )==0) {
                        if(millis()-sustain_time>=_sid_sustain_decay_durations[current_instruments[voice]->release]) {
                            sid.setFrequency(voice,0);
                            _sid_voices_busy[voice]=false;
                            break;
                        } else {
                            current_instruments[voice]->after_off(voice,current_instruments[voice]->current_note);
                            vTaskDelay(1);
                        }
                    }
                    _sid_voices_busy[voice]=false;
                    break;
                }
            }
        }
    }
};


static uint16_t sample1[] = {
// this is ugly but it does not make text editors hang
0x0018,0x0000,0x0000, 0x0016,0x0000,0x0000, 0x0015,0x0000,0x11a0, 0x0006,0x0000,0x00c1, 0x0005,0x0000,0x000a, 0x0004,0x0000,0x000a, 0x0003,0x0000,0x000a, 0x0002,0x0000,0x000a, 0x0001,0x0000,0x000a, 0x0000,0x0000,0x000a,
0x0017,0x0008,0x000a, 0x0017,0x0000,0x0031, 0x0016,0x0008,0x0007, 0x0016,0x0000,0x000c, 0x0015,0x0008,0x0007, 0x0015,0x0000,0x000c, 0x0006,0x0008,0x013c, 0x0006,0x0000,0x000c, 0x0005,0x0008,0x0007, 0x0005,0x0000,0x000c,
0x0004,0x0008,0x0007, 0x0004,0x0000,0x000c, 0x0003,0x0008,0x0007, 0x0003,0x0000,0x000c, 0x0002,0x0008,0x0007, 0x0002,0x0000,0x000c, 0x0001,0x0008,0x0007, 0x0001,0x0000,0x000c, 0x0000,0x0008,0x0007, 0x0000,0x0000,0x000c,
0x0018,0x007f,0x0007, 0x0017,0x00f1,0x2ed6, 0x0018,0x001f,0x021f, 0x0015,0x0004,0x0009, 0x0016,0x0089,0x022a, 0x0000,0x001f,0x001e, 0x0001,0x001f,0x001d, 0x0002,0x0000,0x0004, 0x0003,0x0008,0x000a, 0x0005,0x0006,0x0004,
0x0006,0x00fa,0x0007, 0x0004,0x0041,0x0007, 0x0015,0x0006,0x000d, 0x0016,0x0088,0x06e9, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x0008,0x3746, 0x0000,0x0029,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004,
0x0016,0x0088,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0010,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x0033,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0087,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012,
0x0003,0x0008,0x1a14, 0x0000,0x003d,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0086,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x0047,0x0004,
0x0001,0x001f,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0085,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x0008,0x1039, 0x0000,0x0051,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0085,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x0030,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x005b,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0084,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x0008,0x1a14,
0x0000,0x0065,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0083,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x006f,0x0004, 0x0001,0x001f,0x003c,
0x0015,0x0006,0x0004, 0x0016,0x0082,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x0008,0x1039, 0x0000,0x0079,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0082,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x0050,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x0083,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0081,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x008d,0x0004,
0x0001,0x001f,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0080,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x0097,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0006,0x0004,
0x0016,0x007f,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x0008,0x1039, 0x0000,0x00a1,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x007f,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0070,0x003d,
0x0003,0x0008,0x0a44, 0x0000,0x00ab,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x007e,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x00b5,0x0004, 0x0001,0x001f,0x003c,
0x0015,0x0004,0x0004, 0x0016,0x007d,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x00bf,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x007c,0x00e6,
0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x0008,0x1039, 0x0000,0x00c9,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x007c,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0090,0x003d, 0x0003,0x0008,0x0a44,
0x0000,0x00d3,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x007b,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x00dd,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0004,0x0004,
0x0016,0x007a,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x00e7,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0079,0x00e6, 0x0018,0x007f,0x001e,
0x0002,0x00a8,0x0012, 0x0003,0x0008,0x1039, 0x0000,0x00f1,0x0004, 0x0001,0x001f,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0079,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00b0,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x00fb,0x0004,
0x0001,0x001f,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0078,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x0005,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0077,0x00bb,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x000f,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0076,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012,
0x0003,0x0008,0x1039, 0x0000,0x0019,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0076,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d0,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x0023,0x0004, 0x0001,0x0020,0x003c,
0x0015,0x0002,0x0004, 0x0016,0x0075,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x002d,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0074,0x00bb, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x00e0,0x12ad, 0x0003,0x0008,0x0078, 0x0000,0x0037,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0073,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x0008,0x1039,
0x0000,0x0041,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0073,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f0,0x003d, 0x0003,0x0008,0x0a44, 0x0000,0x004b,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0002,0x0004,
0x0016,0x0072,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x0008,0x1a14, 0x0000,0x0055,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0071,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x0000,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x005f,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0070,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0069,0x0004,
0x0001,0x0020,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0070,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0010,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x0073,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x006f,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x007d,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x006e,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x12ad,
0x0003,0x0009,0x0078, 0x0000,0x0087,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x006d,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0091,0x0004, 0x0001,0x0020,0x003c,
0x0015,0x0000,0x0004, 0x0016,0x006d,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0030,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x009b,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x006c,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x0038,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x00a5,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x006b,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x12ad, 0x0003,0x0009,0x0078,
0x0000,0x00af,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x006a,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x00b9,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0000,0x0004,
0x0016,0x006a,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0050,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x00c3,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0069,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012,
0x0003,0x0009,0x1a14, 0x0000,0x00cd,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0068,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x00d7,0x0004,
0x0001,0x0020,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0067,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x00e1,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0067,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x0070,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x00eb,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0066,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x0009,0x1a14,
0x0000,0x00f5,0x0004, 0x0001,0x0020,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0065,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x00ff,0x0004, 0x0001,0x0020,0x003c,
0x0015,0x0006,0x0004, 0x0016,0x0064,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0009,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0064,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x0090,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x0013,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0063,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x001d,0x0004,
0x0001,0x0021,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0062,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x0027,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004,
0x0016,0x0061,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0031,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0061,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00b0,0x003d,
0x0003,0x0009,0x0a44, 0x0000,0x003b,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0060,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x0045,0x0004, 0x0001,0x0021,0x003c,
0x0015,0x0004,0x0004, 0x0016,0x005f,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x004f,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x005e,0x00e6,
0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0059,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x005e,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d0,0x003d, 0x0003,0x0009,0x0a44,
0x0000,0x0063,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x005d,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x006d,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0004,0x0004,
0x0016,0x005c,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x12ad, 0x0003,0x0009,0x0078, 0x0000,0x0077,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x005b,0x00e6, 0x0018,0x007f,0x001e,
0x0002,0x00e8,0x0012, 0x0003,0x0009,0x1039, 0x0000,0x0081,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x005b,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f0,0x003d, 0x0003,0x0009,0x0a44, 0x0000,0x008b,0x0004,
0x0001,0x0021,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x005a,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x0009,0x1a14, 0x0000,0x0095,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0059,0x00bb,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x009f,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0058,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012,
0x0003,0x000a,0x1039, 0x0000,0x00a9,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0058,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0010,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x00b3,0x0004, 0x0001,0x0021,0x003c,
0x0015,0x0002,0x0004, 0x0016,0x0057,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x00bd,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0056,0x00bb, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x0020,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x00c7,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0055,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x000a,0x1039,
0x0000,0x00d1,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0055,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0030,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x00db,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0002,0x0004,
0x0016,0x0054,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x00e5,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0053,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x0040,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x00ef,0x0004, 0x0001,0x0021,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0052,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x00f9,0x0004,
0x0001,0x0021,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0052,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0050,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x0003,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0051,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x000d,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0050,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12ad,
0x0003,0x000a,0x0078, 0x0000,0x0017,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x004f,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x0021,0x0004, 0x0001,0x0022,0x003c,
0x0015,0x0000,0x0004, 0x0016,0x004f,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0070,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x002b,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x004e,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x0078,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x0035,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x004d,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x12ad, 0x0003,0x000a,0x0078,
0x0000,0x003f,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x004c,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x0049,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0000,0x0004,
0x0016,0x004c,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0090,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x0053,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x004b,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012,
0x0003,0x000a,0x1a14, 0x0000,0x005d,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x004a,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x0067,0x0004,
0x0001,0x0022,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0049,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0049,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x00b0,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x007b,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0048,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x000a,0x1a14,
0x0000,0x0085,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0047,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x008f,0x0004, 0x0001,0x0022,0x003c,
0x0015,0x0006,0x0004, 0x0016,0x0046,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x0099,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0046,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x00d0,0x003d, 0x0003,0x000a,0x0a44, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0045,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x00ad,0x0004,
0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0044,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x12ad, 0x0003,0x000a,0x0078, 0x0000,0x00b7,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0006,0x0004,
0x0016,0x0043,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x000a,0x1039, 0x0000,0x00c1,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0043,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f0,0x003d,
0x0003,0x000a,0x0a44, 0x0000,0x00cb,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0042,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x000a,0x1a14, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x003c,
0x0015,0x0004,0x0004, 0x0016,0x0041,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x12ad, 0x0003,0x000b,0x0078, 0x0000,0x00df,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0040,0x00e6,
0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x000b,0x1039, 0x0000,0x00e9,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0040,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0010,0x003d, 0x0003,0x000b,0x0a44,
0x0000,0x00f3,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x003f,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x000b,0x1a14, 0x0000,0x00fd,0x0004, 0x0001,0x0022,0x003c, 0x0015,0x0004,0x0004,
0x0016,0x003e,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x12ad, 0x0003,0x000b,0x0078, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x003d,0x00e6, 0x0018,0x007f,0x001e,
0x0002,0x0028,0x0012, 0x0003,0x000b,0x1039, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x003d,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0030,0x003d, 0x0003,0x000b,0x0a44, 0x0000,0x0039,0x0004,
0x0001,0x0023,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x003c,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x000b,0x1a14, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x003b,0x00bb,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x12ad, 0x0003,0x000b,0x0078, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x003a,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012,
0x0003,0x000b,0x1039, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x003a,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0050,0x003d, 0x0003,0x000b,0x0a44, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x003c,
0x0015,0x0002,0x0004, 0x0016,0x0039,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x000b,0x1a14, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0038,0x00bb, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x0060,0x12ad, 0x0003,0x000b,0x0078, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0037,0x00e6, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x000b,0x1039,
0x0000,0x00e8,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x0037,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0070,0x003d, 0x0003,0x000b,0x0a44, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004,
0x0016,0x0036,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x000b,0x1a0e, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0035,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x0080,0x12a7, 0x0003,0x000b,0x0078, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x0034,0x00e5, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x000b,0x1030, 0x0000,0x0084,0x0004,
0x0001,0x0023,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0034,0x00bb, 0x0018,0x007f,0x0049, 0x0002,0x0090,0x0012, 0x0003,0x000b,0x0a3e, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0033,0x00bb,
0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x000b,0x1a0e, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0032,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x12a7,
0x0003,0x000b,0x0078, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x0031,0x00e5, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x000b,0x1030, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0043,
0x0015,0x0000,0x0004, 0x0016,0x0031,0x00bb, 0x0018,0x007f,0x0049, 0x0002,0x00b0,0x0012, 0x0003,0x000b,0x0a3e, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0030,0x00bb, 0x0018,0x007f,0x001e,
0x0002,0x00b8,0x0012, 0x0003,0x000b,0x1a0e, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x002f,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x12a7, 0x0003,0x000b,0x0078,
0x0000,0x00d5,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x002e,0x00e5, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x000b,0x1030, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004,
0x0016,0x002e,0x00bb, 0x0018,0x007f,0x0049, 0x0002,0x00d0,0x0012, 0x0003,0x000b,0x0a3e, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x002d,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012,
0x0003,0x000b,0x1a0e, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x002c,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x12a7, 0x0003,0x000b,0x0078, 0x0000,0x0071,0x0004,
0x0001,0x0022,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x002b,0x00e5, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x000b,0x1030, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x002b,0x00bb,
0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x000b,0x0a3e, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x002a,0x00bb, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x000b,0x1a0e,
0x0000,0x0026,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0029,0x00bb, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x12a7, 0x0003,0x000c,0x0078, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x004a,
0x0015,0x0003,0x0004, 0x0016,0x0029,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x000c,0x1024, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0002,0x0004, 0x0016,0x0029,0x00c1, 0x0018,0x007f,0x0049,
0x0002,0x0010,0x0012, 0x0003,0x000c,0x0a2f, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0001,0x0004, 0x0016,0x0029,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x000c,0x1a02, 0x0000,0x008a,0x0004,
0x0001,0x0022,0x004a, 0x0015,0x0000,0x0004, 0x0016,0x0029,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x1297, 0x0003,0x000c,0x0078, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0007,0x0004,
0x0016,0x0028,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x000c,0x1024, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0006,0x0004, 0x0016,0x0028,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0030,0x0012,
0x0003,0x000c,0x0a32, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0005,0x0004, 0x0016,0x0028,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x000c,0x19ff, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x004a,
0x0015,0x0004,0x0004, 0x0016,0x0028,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x129b, 0x0003,0x000c,0x0078, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x004a, 0x0015,0x0003,0x0004, 0x0016,0x0028,0x00ec,
0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x000c,0x1024, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0094, 0x0015,0x0002,0x0004, 0x0016,0x0028,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0050,0x0012, 0x0003,0x000c,0x09e7,
0x0000,0x0039,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0001,0x0004, 0x0016,0x0028,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x000c,0x1a0e, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004,
0x0016,0x0028,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12a7, 0x0003,0x000c,0x0078, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0007,0x0004, 0x0016,0x0027,0x00ec, 0x0018,0x007f,0x001e,
0x0002,0x0068,0x0012, 0x0003,0x000c,0x1030, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0027,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0070,0x003c, 0x0003,0x000c,0x0a41, 0x0000,0x009d,0x0004,
0x0001,0x0023,0x003c, 0x0015,0x0005,0x0004, 0x0016,0x0027,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x000c,0x1a0e, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x0027,0x00c1,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x12a6, 0x0003,0x000c,0x0078, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0003,0x0004, 0x0016,0x0027,0x00eb, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012,
0x0003,0x000c,0x1033, 0x0000,0x00e8,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0027,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0090,0x003c, 0x0003,0x000c,0x0a41, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x0043,
0x0015,0x0001,0x0004, 0x0016,0x0027,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x000c,0x1a08, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0027,0x00c1, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x00a0,0x129e, 0x0003,0x000c,0x0078, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x0026,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x000c,0x102d,
0x0000,0x0084,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x0026,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00b0,0x0012, 0x0003,0x000c,0x0a38, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0005,0x0004,
0x0016,0x0026,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x000c,0x1a05, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0026,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x00c0,0x12a0, 0x0003,0x000c,0x0078, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x0026,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x000c,0x102d, 0x0000,0x0020,0x0004,
0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0026,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00d0,0x0012, 0x0003,0x000c,0x0a38, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x0026,0x00c1,
0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x000c,0x1a05, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0026,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x12a0,
0x0003,0x000c,0x0078, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x0025,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x000c,0x102d, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x0043,
0x0015,0x0006,0x0004, 0x0016,0x0025,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x000c,0x0a38, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0005,0x0004, 0x0016,0x0025,0x00c1, 0x0018,0x007f,0x001e,
0x0002,0x00f8,0x0012, 0x0003,0x000c,0x1a05, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0025,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x12a0, 0x0003,0x000d,0x0078,
0x0000,0x0071,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x0025,0x02f6, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x000c,0x0e29, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0002,0x0004,
0x0016,0x0025,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x000c,0x0a39, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x0025,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012,
0x0003,0x000c,0x1a06, 0x0000,0x0026,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0025,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x1298, 0x0003,0x000c,0x007f, 0x0000,0x003f,0x0004,
0x0001,0x0022,0x004a, 0x0015,0x0007,0x0004, 0x0016,0x0024,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x000c,0x1025, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0006,0x0004, 0x0016,0x0024,0x00c1,
0x0018,0x007f,0x0049, 0x0002,0x00d0,0x0012, 0x0003,0x000c,0x0a30, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0005,0x0004, 0x0016,0x0024,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x000c,0x1a00,
0x0000,0x008a,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0004,0x0004, 0x0016,0x0024,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x1292, 0x0003,0x000c,0x007f, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x004a,
0x0015,0x0003,0x0004, 0x0016,0x0024,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x000c,0x1025, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0002,0x0004, 0x0016,0x0024,0x00c1, 0x0018,0x007f,0x0049,
0x0002,0x00b0,0x0012, 0x0003,0x000c,0x0a30, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0001,0x0004, 0x0016,0x0024,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x000c,0x1a00, 0x0000,0x00ee,0x0004,
0x0001,0x0022,0x004a, 0x0015,0x0000,0x0004, 0x0016,0x0024,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x1292, 0x0003,0x000c,0x007f, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x004a, 0x0015,0x0007,0x0004,
0x0016,0x0023,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x000c,0x1025, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0094, 0x0015,0x0006,0x0004, 0x0016,0x0023,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0090,0x0012,
0x0003,0x000c,0x09e8, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0005,0x0004, 0x0016,0x0023,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x000c,0x1a0c, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x003c,
0x0015,0x0004,0x0004, 0x0016,0x0023,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x12a0, 0x0003,0x000c,0x007f, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0003,0x0004, 0x0016,0x0023,0x00ec,
0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x000c,0x1034, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x0023,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0070,0x0012, 0x0003,0x000c,0x0a3f,
0x0000,0x009d,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0001,0x0004, 0x0016,0x0023,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x000c,0x1a0c, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004,
0x0016,0x0023,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12a0, 0x0003,0x000c,0x007f, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0007,0x0004, 0x0016,0x0022,0x00ec, 0x0018,0x007f,0x001e,
0x0002,0x0058,0x0012, 0x0003,0x000c,0x1034, 0x0000,0x00e8,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0022,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0050,0x0012, 0x0003,0x000c,0x0a3f, 0x0000,0x00cf,0x0004,
0x0001,0x0023,0x0043, 0x0015,0x0005,0x0004, 0x0016,0x0022,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x000c,0x1a06, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0022,0x00c1,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x1298, 0x0003,0x000c,0x007f, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x0022,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012,
0x0003,0x000c,0x102b, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0022,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0030,0x0012, 0x0003,0x000c,0x0a39, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x0043,
0x0015,0x0001,0x0004, 0x0016,0x0022,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x000c,0x1a06, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0022,0x00c1, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x0020,0x129b, 0x0003,0x000c,0x007f, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x0021,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x000c,0x102b,
0x0000,0x0020,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x0021,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0010,0x0012, 0x0003,0x000c,0x0a36, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0005,0x0004,
0x0016,0x0021,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x000c,0x1a09, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0021,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x0000,0x1297, 0x0003,0x000c,0x007f, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x0021,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x000b,0x102b, 0x0000,0x00bc,0x0004,
0x0001,0x0022,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0021,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x000b,0x0a39, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x0021,0x00c1,
0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x000b,0x1a06, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x0021,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x129b,
0x0003,0x000b,0x007f, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x0020,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x000b,0x102b, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x0043,
0x0015,0x0006,0x0004, 0x0016,0x0020,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00d0,0x0012, 0x0003,0x000b,0x0a36, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0005,0x0004, 0x0016,0x0020,0x00c1, 0x0018,0x007f,0x001e,
0x0002,0x00c8,0x0012, 0x0003,0x000b,0x1a09, 0x0000,0x0026,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x0020,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00c0,0x1297, 0x0003,0x000b,0x007f,
0x0000,0x003f,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0003,0x0004, 0x0016,0x0020,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x000b,0x1025, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0002,0x0004,
0x0016,0x0020,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00b0,0x0012, 0x0003,0x000b,0x0a30, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0001,0x0004, 0x0016,0x0020,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012,
0x0003,0x000b,0x1a00, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0000,0x0004, 0x0016,0x0020,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x1292, 0x0003,0x000b,0x007f, 0x0000,0x00a3,0x0004,
0x0001,0x0022,0x004a, 0x0015,0x0007,0x0004, 0x0016,0x001f,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x000b,0x1025, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0006,0x0004, 0x0016,0x001f,0x00c1,
0x0018,0x007f,0x0049, 0x0002,0x0090,0x0012, 0x0003,0x000b,0x0a30, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0005,0x0004, 0x0016,0x001f,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0088,0x0012, 0x0003,0x000b,0x1a00,
0x0000,0x00ee,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0004,0x0004, 0x0016,0x001f,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x1292, 0x0003,0x000b,0x007f, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x004a,
0x0015,0x0003,0x0004, 0x0016,0x001f,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x000b,0x1025, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0094, 0x0015,0x0002,0x0004, 0x0016,0x001f,0x00ec, 0x0018,0x007f,0x001e,
0x0002,0x0070,0x0012, 0x0003,0x000b,0x09e8, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0001,0x0004, 0x0016,0x001f,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012, 0x0003,0x000b,0x1a0c, 0x0000,0x0052,0x0004,
0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x001f,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x12a0, 0x0003,0x000b,0x007f, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0007,0x0004,
0x0016,0x001e,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x000b,0x1034, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x001e,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0050,0x0012,
0x0003,0x000b,0x0a3f, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0005,0x0004, 0x0016,0x001e,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x000b,0x1a0c, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x003c,
0x0015,0x0004,0x0004, 0x0016,0x001e,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x12a0, 0x0003,0x000b,0x007f, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0003,0x0004, 0x0016,0x001e,0x00ec,
0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x000b,0x1034, 0x0000,0x00e8,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x001e,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0030,0x0012, 0x0003,0x000b,0x0a3f,
0x0000,0x00cf,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x001e,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x000b,0x1a06, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0000,0x0004,
0x0016,0x001e,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x1298, 0x0003,0x000b,0x007f, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x001d,0x00ec, 0x0018,0x007f,0x001e,
0x0002,0x0018,0x0012, 0x0003,0x000b,0x102b, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x001d,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0010,0x0012, 0x0003,0x000b,0x0a39, 0x0000,0x006b,0x0004,
0x0001,0x0023,0x0043, 0x0015,0x0005,0x0004, 0x0016,0x001d,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x000b,0x1a06, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x001d,0x00c1,
0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x129b, 0x0003,0x000b,0x007f, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x001d,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012,
0x0003,0x000a,0x102b, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x001d,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x000a,0x0a36, 0x0000,0x0007,0x0004, 0x0001,0x0023,0x0043,
0x0015,0x0001,0x0004, 0x0016,0x001d,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x000a,0x1a09, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x001d,0x00c1, 0x0018,0x007f,0x001e,
0x0018,0x007f,0x0012, 0x0002,0x00e0,0x1297, 0x0003,0x000a,0x007f, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0007,0x0004, 0x0016,0x001c,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00d8,0x0012, 0x0003,0x000a,0x102b,
0x0000,0x00bc,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0006,0x0004, 0x0016,0x001c,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00d0,0x0012, 0x0003,0x000a,0x0a39, 0x0000,0x00a3,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0005,0x0004,
0x0016,0x001c,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00c8,0x0012, 0x0003,0x000a,0x1a06, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0004,0x0004, 0x0016,0x001c,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012,
0x0002,0x00c0,0x129b, 0x0003,0x000a,0x007f, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x001c,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x00b8,0x0012, 0x0003,0x000a,0x102b, 0x0000,0x0058,0x0004,
0x0001,0x0022,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x001c,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00b0,0x0012, 0x0003,0x000a,0x0a36, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x001c,0x00c1,
0x0018,0x007f,0x001e, 0x0002,0x00a8,0x0012, 0x0003,0x000a,0x1a09, 0x0000,0x0026,0x0004, 0x0001,0x0022,0x0043, 0x0015,0x0000,0x0004, 0x0016,0x001c,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00a0,0x1297,
0x0003,0x000a,0x007f, 0x0000,0x003f,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0007,0x0004, 0x0016,0x001b,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0098,0x0012, 0x0003,0x000a,0x1025, 0x0000,0x0058,0x0004, 0x0001,0x0022,0x004a,
0x0015,0x0006,0x0004, 0x0016,0x001b,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0090,0x0012, 0x0003,0x000a,0x0a30, 0x0000,0x0071,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0005,0x0004, 0x0016,0x001b,0x00c1, 0x0018,0x007f,0x001e,
0x0002,0x0088,0x0012, 0x0003,0x000a,0x1a00, 0x0000,0x008a,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0004,0x0004, 0x0016,0x001b,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0080,0x1292, 0x0003,0x000a,0x007f,
0x0000,0x00a3,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0003,0x0004, 0x0016,0x001b,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0078,0x0012, 0x0003,0x000a,0x1025, 0x0000,0x00bc,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0002,0x0004,
0x0016,0x001b,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0070,0x0012, 0x0003,0x000a,0x0a30, 0x0000,0x00d5,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0001,0x0004, 0x0016,0x001b,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0068,0x0012,
0x0003,0x000a,0x1a00, 0x0000,0x00ee,0x0004, 0x0001,0x0022,0x004a, 0x0015,0x0000,0x0004, 0x0016,0x001b,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0060,0x1292, 0x0003,0x000a,0x007f, 0x0000,0x0007,0x0004,
0x0001,0x0023,0x004a, 0x0015,0x0007,0x0004, 0x0016,0x001a,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0058,0x0012, 0x0003,0x000a,0x1025, 0x0000,0x0020,0x0004, 0x0001,0x0023,0x0094, 0x0015,0x0006,0x0004, 0x0016,0x001a,0x00ec,
0x0018,0x007f,0x001e, 0x0002,0x0050,0x0012, 0x0003,0x000a,0x09e8, 0x0000,0x0039,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0005,0x0004, 0x0016,0x001a,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0048,0x0012, 0x0003,0x000a,0x1a0c,
0x0000,0x0052,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0004,0x0004, 0x0016,0x001a,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0040,0x12a0, 0x0003,0x000a,0x007f, 0x0000,0x006b,0x0004, 0x0001,0x0023,0x003c,
0x0015,0x0003,0x0004, 0x0016,0x001a,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0038,0x0012, 0x0003,0x000a,0x1034, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0002,0x0004, 0x0016,0x001a,0x00c1, 0x0018,0x007f,0x0049,
0x0002,0x0030,0x0012, 0x0003,0x000a,0x0a3f, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0001,0x0004, 0x0016,0x001a,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0028,0x0012, 0x0003,0x000a,0x1a0c, 0x0000,0x00b6,0x0004,
0x0001,0x0023,0x003c, 0x0015,0x0000,0x0004, 0x0016,0x001a,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0020,0x12a0, 0x0003,0x000a,0x007f, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0007,0x0004,
0x0016,0x0019,0x00ec, 0x0018,0x007f,0x001e, 0x0002,0x0018,0x0012, 0x0003,0x000a,0x1034, 0x0000,0x00e8,0x0004, 0x0001,0x0023,0x003c, 0x0015,0x0006,0x0004, 0x0016,0x0019,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x0010,0x0012,
0x0003,0x000a,0x0a3f, 0x0000,0x00cf,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0005,0x0004, 0x0016,0x0019,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x0008,0x0012, 0x0003,0x000a,0x1a06, 0x0000,0x00b6,0x0004, 0x0001,0x0023,0x0043,
0x0015,0x0004,0x0004, 0x0016,0x0019,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x0000,0x1298, 0x0003,0x000a,0x007f, 0x0000,0x009d,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0003,0x0004, 0x0016,0x0019,0x00ec,
0x0018,0x007f,0x001e, 0x0002,0x00f8,0x0012, 0x0003,0x0009,0x102b, 0x0000,0x0084,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0002,0x0004, 0x0016,0x0019,0x00c1, 0x0018,0x007f,0x0049, 0x0002,0x00f0,0x0012, 0x0003,0x0009,0x0a39,
0x0000,0x006b,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0001,0x0004, 0x0016,0x0019,0x00c1, 0x0018,0x007f,0x001e, 0x0002,0x00e8,0x0012, 0x0003,0x0009,0x1a06, 0x0000,0x0052,0x0004, 0x0001,0x0023,0x0043, 0x0015,0x0000,0x0004,
0x0016,0x0019,0x00c1, 0x0018,0x007f,0x001e, 0x0018,0x007f,0x0012, 0x0002,0x00e0,0x129b, 0x0003,0x0009,0x007f, 0x0000,0x0039

};

static uint16_t sample2[] = {

0x0018,0x0000,0x0000, 0x0000,0x0000,0x0000, 0x0001,0x0000,0x0000, 0x0002,0x0000,0x0000, 0x0003,0x0000,0x0000, 0x0004,0x0000,0x0000, 0x0005,0x0000,0x0000, 0x0006,0x0000,0x0000, 0x0015,0x0000,0x0000, 0x0016,0x0000,0x0000,
0x0017,0x0000,0x0000, 0x0018,0x0000,0x0000, 0x0019,0x0000,0x0000, 0x001a,0x0000,0x0000, 0x0018,0x000f,0x0000, 0x0016,0x0020,0x0000, 0x0016,0x0020,0x0000, 0x0002,0x00c0,0x0000, 0x0003,0x0007,0x0000, 0x0016,0x001e,0x0000,
0x0000,0x0075,0x0000, 0x0001,0x0006,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x001e,0x0000, 0x0016,0x001e,0x0000, 0x0002,0x00f0,0x0000, 0x0003,0x0007,0x0000, 0x0016,0x0020,0x0000, 0x0000,0x0055,0x0000, 0x0001,0x0006,0x0000,
0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000, 0x0017,0x00f1,0x0000, 0x0005,0x0000,0x0000, 0x0006,0x00a4,0x0000, 0x0004,0x0040,0x0000, 0x0018,0x001f,0x0000, 0x0017,0x00f1,0x0000, 0x0004,0x0009,0x0000, 0x0016,0x0020,0x0000,
0x0016,0x0020,0x0000, 0x0002,0x0020,0x0000, 0x0003,0x0002,0x0000, 0x0016,0x0041,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0041,0x0000, 0x0016,0x0041,0x0000, 0x0002,0x0050,0x0000,
0x0003,0x0002,0x0000, 0x0016,0x0042,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0042,0x0000, 0x0016,0x0042,0x0000, 0x0002,0x0080,0x0000, 0x0003,0x0002,0x0000, 0x0016,0x0043,0x0000,
0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000, 0x0016,0x0043,0x0000, 0x0002,0x00b0,0x0000, 0x0003,0x0002,0x0000, 0x0016,0x0044,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000,
0x0004,0x0041,0x0000, 0x0016,0x0044,0x0000, 0x0016,0x0044,0x0000, 0x0002,0x00e0,0x0000, 0x0003,0x0002,0x0000, 0x0016,0x0045,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0045,0x0000,
0x0016,0x0045,0x0000, 0x0002,0x0010,0x0000, 0x0003,0x0003,0x0000, 0x0016,0x0046,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0046,0x0000, 0x0016,0x0046,0x0000, 0x0002,0x0040,0x0000,
0x0003,0x0003,0x0000, 0x0016,0x0047,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000, 0x0016,0x0047,0x0000, 0x0002,0x0070,0x0000, 0x0003,0x0003,0x0000, 0x0016,0x0048,0x0000,
0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0048,0x0000, 0x0016,0x0048,0x0000, 0x0002,0x00a0,0x0000, 0x0003,0x0003,0x0000, 0x0016,0x0049,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000,
0x0004,0x0041,0x0000, 0x0016,0x0049,0x0000, 0x0016,0x0049,0x0000, 0x0002,0x00d0,0x0000, 0x0003,0x0003,0x0000, 0x0016,0x004a,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x004a,0x0000,
0x0016,0x004a,0x0000, 0x0002,0x0000,0x0000, 0x0003,0x0004,0x0000, 0x0016,0x004b,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000, 0x0016,0x004b,0x0000, 0x0002,0x0030,0x0000,
0x0003,0x0004,0x0000, 0x0016,0x004c,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x004c,0x0000, 0x0016,0x004c,0x0000, 0x0002,0x0060,0x0000, 0x0003,0x0004,0x0000, 0x0016,0x004d,0x0000,
0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x004d,0x0000, 0x0016,0x004d,0x0000, 0x0002,0x0090,0x0000, 0x0003,0x0004,0x0000, 0x0016,0x004e,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000,
0x0004,0x0041,0x0000, 0x0016,0x004e,0x0000, 0x0016,0x004e,0x0000, 0x0002,0x00c0,0x0000, 0x0003,0x0004,0x0000, 0x0016,0x004f,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000,
0x0016,0x004f,0x0000, 0x0002,0x00f0,0x0000, 0x0003,0x0004,0x0000, 0x0016,0x0050,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0050,0x0000, 0x0016,0x0050,0x0000, 0x0002,0x0020,0x0000,
0x0003,0x0005,0x0000, 0x0016,0x0051,0x0000, 0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0051,0x0000, 0x0016,0x0051,0x0000, 0x0002,0x0050,0x0000, 0x0003,0x0005,0x0000, 0x0016,0x0052,0x0000,
0x0000,0x00a3,0x0000, 0x0001,0x000e,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0052,0x0000, 0x0016,0x0052,0x0000, 0x0002,0x0080,0x0000, 0x0003,0x0005,0x0000, 0x0016,0x0051,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000,
0x0004,0x0041,0x0000, 0x0017,0x00f1,0x0000, 0x0016,0x0051,0x0000, 0x0002,0x00b0,0x0000, 0x0003,0x0005,0x0000, 0x0016,0x0050,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x0050,0x0000,
0x0016,0x0050,0x0000, 0x0002,0x00e0,0x0000, 0x0003,0x0005,0x0000, 0x0016,0x004f,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000, 0x0016,0x004f,0x0000, 0x0016,0x004f,0x0000, 0x0002,0x0010,0x0000,
0x0003,0x0006,0x0000, 0x0016,0x004e,0x0000, 0x0000,0x0051,0x0000, 0x0001,0x0007,0x0000, 0x0004,0x0041,0x0000,
};



class sid_piano5 : public sid_instrument {
  public:
    int i;
    int flo,fhi,plo,phi;
    sid_piano5() {
        df2=sample2;
    }
    virtual void start_sample(int voice,int note) {
        i=30;
    }
    uint16_t *df2;
    virtual void next_instruction(int voice,int note) {
        i=(i+1)%187;
        if(i==0)
            i=43;
        switch(df2[i*3]) {
            case 0:
                flo=df2[i*3+1];
            break;
            case 1:
                fhi=df2[i*3+1];
                sid.setFrequency(voice,((fhi*256+flo)*note)/3747);
            break;
            default:
                if(df2[i*3]<7)
                    sid.pushRegister(voice/3,df2[i*3]+(voice%3)*7,df2[i*3+1]);
            break;
        }

        vTaskDelay(df2[i*3+2]/1000+2);
    }
    virtual void after_off(int voice,int note){
        i=30;
    }
};


class sid_piano : public sid_instrument {
  public:
    int i;
    int flo,fhi,plo,phi;
    sid_piano() {
        df2=sample1;
    }
    virtual void start_sample(int voice,int note) {
        i=0;
    }
    uint16_t *df2;
    virtual void next_instruction(int voice,int note) {
        i=(i+1)%1918;
        if(i==0)
            i=1651;
        switch(df2[i*3]) {
            case 0:
                flo=df2[i*3+1];
            break;
            case 1:
                fhi=df2[i*3+1];
                sid.setFrequency(voice,((fhi*256+flo)*note/7977));
            break;
            default:
                if(df2[i*3]<7)
                    sid.pushRegister(voice/3,df2[i*3]+(voice%3)*7,df2[i*3+1]);
            break;
        }
        vTaskDelay(df2[i*3+2]/1000);
    }
};



class sid_piano4 : public sid_instrument {
  public:
    int i;
    virtual void start_sample(int voice,int note) {
        sid.setAttack(voice,3);
        sid.setSustain(voice,9);
        sid.setDecay(voice,9);
        sid.setRelease(voice,10);
        sid.setPulse(voice,3000);
        sid.setWaveForm(voice,SID_WAVEFORM_SAWTOOTH);
        sid.setFrequency(voice,note);
        sid.setGate(voice,1);
        i=0;
    }
    virtual void next_instruction(int voice,int note) {
        sid.setFrequency(voice,note+100*(cos(2*3.14*i/10)));
        //sid.setGate(voice,(i/3)%2);
        //x
        i=(i+1)%50;
        vTaskDelay(20);
    }
    virtual void after_off(int voice,int note) {
        sid.setFrequency(voice,note+100*(cos(2*3.14*i/10)));
        i=(i+1)%50;
        vTaskDelay(20);
    }
};


class sid_piano3 : public sid_instrument {
  public:
    int i;
    virtual void start_sample(int voice,int note) {
        sid.setAttack(voice,0);
        sid.setSustain(voice,1);
        sid.setDecay(voice,10);
        sid.setRelease(voice,10);
        sid.setPulse(voice,3000);
        sid.setWaveForm(voice,SID_WAVEFORM_SAWTOOTH);
        sid.setFrequency(voice,note);
        sid.setGate(voice,1);
    }
};


class sid_piano2 : public sid_instrument {
  public:
    int i;
    virtual void start_sample(int voice,int note) {
        sid.setAttack(voice,0);
        sid.setSustain(voice,15);
        sid.setDecay(voice,2);
        sid.setRelease(voice,10);
        sid.setPulse(voice,1000);
        sid.setWaveForm(voice,SID_WAVEFORM_PULSE);
        sid.setGate(voice,1);
        i=0;
    }
    virtual void next_instruction(int voice,int note) {
        //log_v(note+10*(cos(2*3.14*i/100)));
        sid.setFrequency(voice,note+20*(1+i/50)*(cos(2*3.14*i/10)));
        //x
        i++;
        vTaskDelay(20);
    }
    virtual void after_off(int voice,int note) {
        sid.setFrequency(voice,note+20*(cos(2*3.14*i/10)));
        //x
        i++;
        vTaskDelay(20);
    }
};


class SIDKeyBoardPlayer {
  public:
    static void KeyBoardPlayer(int nbofvoices) {
        sid.resetsid();
        createnot();
        keyboardnbofvoices=(nbofvoices%15)+1;
        for(int i=0;i<nbofvoices;i++) {
            _sid_taskready_busy[i]=false;
        }
        for(int i=0;i<nbofvoices;i++) {
            _sid_voicesQueues[i]= xQueueCreate( 100, sizeof( _sid_command ) );
            _sid_voices_busy[i]=false;
        }
        if(nbofvoices>=1) {
            xTaskCreate( VoiceTask<0>::vTaskCode, "NAME1",  2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[0] );
        }
        if(nbofvoices>=2) {
            xTaskCreate( VoiceTask<1>::vTaskCode, "NAME2", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[1] );
        }
        if(nbofvoices>=3) {
            xTaskCreate( VoiceTask<2>::vTaskCode, "NAME3", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[2] );
        }
        if(nbofvoices>=4) {
            xTaskCreate( VoiceTask<3>::vTaskCode, "NAME4", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[3] );
        }
        if(nbofvoices>=5) {
            xTaskCreate( VoiceTask<4>::vTaskCode, "NAME5", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[4] );
        }
        if(nbofvoices>=6) {
            xTaskCreate( VoiceTask<5>::vTaskCode, "NAME6", 2048,( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[5] );
        }
        if(nbofvoices>=7) {
            xTaskCreate( VoiceTask<6>::vTaskCode, "NAME7", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[6] );
        }
        if(nbofvoices>=8) {
            xTaskCreate( VoiceTask<7>::vTaskCode, "NAME8", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[7] );
        }
        if(nbofvoices==9) {
            xTaskCreate( VoiceTask<8>::vTaskCode, "NAME9", 2048, ( void * ) 1, tskIDLE_PRIORITY, & _sid_xHandletab[8] );
        }
        changeAllInstruments<sid_piano4>();
        while(allTaskReady());
        log_i("Keyboard intiated");
    }

    template <typename instrument>static void changeAllInstruments() {
        for(int i=0;i<keyboardnbofvoices;i++) {
            current_instruments[i]=new instrument;
            sid.voices[i].release=0;
        }
    }

    template <typename instrument>static void changeInstrumentOnVoice(int voice) {
        if(voice<15) {
            current_instruments[voice]=new instrument;
            sid.voices[voice].release=0;
        }

    }

    static void createnot() {
        double f=274.3352163421902;
        double  g=powf((double)2,(double)1/12);
        for(int i=0;i<95;i++) {
            uint16_t d=floor(f);
            _sid_play_notes[i]=d;
            f=f*g;
        }
    }

    static void playNoteVelocity(int voice,uint16_t note,int velocity,uint32_t duration) {
        _sid_command c;
        c.note=note;
        c.duration=duration;
        c.velocity=velocity;
        _sid_voices_busy[voice]=true;
        xQueueSend(_sid_voicesQueues[voice], &c, portMAX_DELAY);
    }

    static void playNote(int voice,uint16_t note,uint32_t duration) {
        _sid_command c;
        c.note=note;
        c.duration=duration;
        c.velocity=120;
        _sid_voices_busy[voice]=true;
        xQueueSend(_sid_voicesQueues[voice], &c, portMAX_DELAY);
    }

    static void stopNote(int voice) {
        _sid_command c;
        c.note=0;
        c.duration=0;
        c.velocity=0;
        _sid_voices_busy[voice]=true;
        xQueueSend(_sid_voicesQueues[voice], &c, portMAX_DELAY);
    }

    static void playNoteHz(int voice,int frequencyHz,uint32_t duration) {
        double fout=frequencyHz*16.777216;
        playNote(voice,(uint16_t)fout,duration);
    }

    static bool isVoiceBusy(int voice) {
        vTaskDelay(1);
        return _sid_voices_busy[voice];
    }

    static bool areAllVoiceBusy() {
        bool res=false;
        vTaskDelay(1);
        for(int i=0;i<keyboardnbofvoices;i++) {
            if( _sid_voices_busy[i]) {
                res=true;
                break;
            }
        }
        return res;
    }

    static bool allTaskReady() {
        vTaskDelay(1);
        for(int i=0;i<keyboardnbofvoices;i++) {
            if( _sid_taskready_busy[i]==false)
                return true;
        }
        return false;
    }
};


#endif /* SID6581_h */
